@@ -29,7 +29,7 @@ package inc::MBX::Alien::FLTK::Platform::Windows;
         $self->notes('define')->{'HAVE_SCANDIR'}       = undef;
         $self->notes('define')->{'HAVE_SCANDIR_POSIX'} = undef;
     GL: {
-            last GL if !$self->find_h('gl.h');
+            last GL if !$self->find_h('GL/gl.h');
             print 'Testing GL Support... ';
             if (!$self->assert_lib({lib => 'opengl32', header => 'GL/gl.h'}))
             {   print "not okay\n";
